@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2005-2008  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2005-2009  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,8 +25,6 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: cpu_arm_instr.c,v 1.77.2.3 2008/06/10 00:18:04 debug Exp $
- *
  *  ARM instructions.
  *
  *  Individual functions should keep track of cpu->n_translated_instrs.
@@ -2801,8 +2799,8 @@ X(to_be_translated)
 
 		/*  "bkpt", ARMv5 and above  */
 		if ((iword & 0x0ff000f0) == 0x01200070) {
-			ic->arg[0] = addr & 0xfff;
 			ic->f = cond_instr(bkpt);
+			ic->arg[0] = addr & 0xfff;
 			break;
 		}
 
@@ -2924,12 +2922,13 @@ X(to_be_translated)
 		else
 			ic->arg[1] = (size_t)(void *)arm_r[iword & 0xfff];
 		if ((iword & 0x0e000010) == 0x06000010) {
-			if (!cpu->translation_readahead)
-				fatal("Not a Load/store TODO\n");
-			goto bad;
+			/*  GDB uses this for breakpoints.  */
+			ic->f = cond_instr(und);
+			ic->arg[0] = addr & 0xfff;
 		}
 		/*  Special case: pc-relative load within the same page:  */
 		if (rn == ARM_PC && rd != ARM_PC && main_opcode < 6 && l_bit) {
+			unsigned char *p = page;
 			int ofs = (addr & 0xfff) + 8, max = 0xffc;
 			int b_bit = iword & 0x00400000;
 			if (b_bit)
@@ -2940,25 +2939,16 @@ X(to_be_translated)
 				ofs -= (iword & 0xfff);
 			/*  NOTE/TODO: This assumes 4KB pages,
 			    it will not work with 1KB pages.  */
-			if (ofs >= 0 && ofs <= max) {
-				unsigned char *p;
+			if (ofs >= 0 && ofs <= max && p != NULL) {
 				unsigned char c[4];
 				int len = b_bit? 1 : 4;
 				uint32_t x, a = (addr & 0xfffff000) | ofs;
 				/*  ic->f = cond_instr(mov);  */
 				ic->f = arm_dpi_instr[condition_code + 16*0xd];
 				ic->arg[2] = (size_t)(&cpu->cd.arm.r[rd]);
-				p = page;
-				if (p != NULL) {
-					memcpy(c, p + (a & 0xfff), len);
-				} else {
-					if (!cpu->memory_rw(cpu, cpu->mem, a, &c[0],
-					    sizeof(c), MEM_READ, CACHE_DATA)) {
-						fatal("Hm? Internal error in "
-						    "cpu_arm_instr.c!\n");
-						goto bad;
-					}
-				}
+
+				memcpy(c, p + (a & 0xfff), len);
+
 				if (b_bit) {
 					x = c[0];
 				} else {
@@ -3134,7 +3124,7 @@ X(to_be_translated)
 		 *  For now, treat as Undefined instructions. This causes e.g.
 		 *  Linux/ARM to emulate these instructions (floating point).
 		 */
-#if 0
+#if 1
 		ic->f = cond_instr(und);
 		ic->arg[0] = addr & 0xfff;
 #else
