@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2003-2008  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2003-2009  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,8 +25,6 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: cpu_mips.c,v 1.84.2.1 2008/01/18 19:12:25 debug Exp $
- *
  *  MIPS core CPU emulation.
  */
 
@@ -724,7 +722,7 @@ int mips_cpu_disassemble_instr(struct cpu *cpu, unsigned char *originstr,
 	else
 		debug("%016"PRIx64, (uint64_t)dumpaddr);
 
-	*((uint32_t *)&instr[0]) = *((uint32_t *)&originstr[0]);
+	memcpy(instr, originstr, sizeof(uint32_t));
 
 	/*
 	 *  The rest of the code is written for little endian,
