@@ -2,7 +2,7 @@
 #define	MACHINE_H
 
 /*
- *  Copyright (C) 2005-2008  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2005-2009  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -26,9 +26,6 @@
  *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  *  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  *  SUCH DAMAGE.
- *
- *
- *  $Id: machine.h,v 1.182.2.2 2008/06/10 00:18:05 debug Exp $
  */
 
 #include <sys/types.h>
@@ -252,7 +249,8 @@ struct machine {
 #define	MACHINE_HPCARM		5003
 #define	MACHINE_NETWINDER	5004
 #define	MACHINE_IQ80321		5005
-#define	MACHINE_QEMU_ARM	5006
+#define	MACHINE_IYONIX		5006
+#define	MACHINE_QEMU_ARM	5007
 
 /*  SH:  */
 #define	MACHINE_BARESH		6000
@@ -265,6 +263,7 @@ struct machine {
 #define	MACHINE_BAREM88K	7000
 #define	MACHINE_TESTM88K	7001
 #define	MACHINE_MVME88K		7002
+#define	MACHINE_LUNA88K		7003
 
 /*  M32R:  */
 #define	MACHINE_BAREM32R	8000
@@ -349,6 +348,10 @@ struct machine {
 #define	MACHINE_MVME88K_188		2
 #define	MACHINE_MVME88K_197		3
 
+/*  LUNA88K  */
+#define	MACHINE_LUNA_88K		1
+#define	MACHINE_LUNA_88K2		2
+
 
 /*  For the automachine system:  */
 struct machine_entry_subtype {
